package br.com.ages.mutiraodobemback.service.event;

import br.com.ages.mutiraodobemback.domain.EventEntity;
import br.com.ages.mutiraodobemback.domain.UserEntity;
import br.com.ages.mutiraodobemback.dto.EventDetailsResponseDTO;
import br.com.ages.mutiraodobemback.dto.EventReponseListDTO;
import br.com.ages.mutiraodobemback.dto.EventRequestDTO;
import br.com.ages.mutiraodobemback.dto.EventResponseDTO;
import br.com.ages.mutiraodobemback.repository.IEventRepository;
import br.com.ages.mutiraodobemback.service.aws.IAwsService;
import br.com.ages.mutiraodobemback.service.user.IUserService;
import br.com.ages.mutiraodobemback.util.Validations;
import br.com.ages.mutiraodobemback.util.exceptions.CustomIllegalArgumentException;
import br.com.ages.mutiraodobemback.util.exceptions.EventNotFoundException;
import br.com.ages.mutiraodobemback.util.exceptions.UserAlreadyParticipate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static br.com.ages.mutiraodobemback.message.Message.*;

@Service
public class EventServiceImpl implements IEventService {

    private final IEventRepository repository;
    private final IUserService userService;
    private final IAwsService awsService;

    @Autowired
    public EventServiceImpl(IEventRepository repository, IUserService userService, IAwsService awsService) {
        this.repository = repository;
        this.userService = userService;
        this.awsService = awsService;
    }

    @Override
    public EventResponseDTO save(EventRequestDTO event) {
        validateFields(event);
        EventEntity eventEntity = new EventEntity(event);
        eventEntity.setCreatedDate(LocalDateTime.now());
        eventEntity.setActive(true);
        eventEntity.setUser(this.userService.getLoggedUser());
        eventEntity = repository.save(eventEntity);
        if (Validations.isNotNull(event.getImages()))
            this.uploadImages(event, eventEntity);

        return new EventResponseDTO(repository.save(eventEntity));
    }

    public EventEntity save(EventEntity event) {
        return repository.save(event);
    }

    @Override
    public EventResponseDTO update(EventRequestDTO event) {
        return this.save(event);
    }

    @Override
    public List<EventRequestDTO> findAll() {
        return repository.findAll().stream().map(EventRequestDTO::new).collect(Collectors.toList());
    }

    @Override
    public List<EventReponseListDTO> findAllActive() {
        return repository.findAllByActiveIsTrue().stream().map(EventReponseListDTO::new).collect(Collectors.toList());
    }

    @Override
    public EventDetailsResponseDTO findById(String id) {
        return new EventDetailsResponseDTO(this.repository.findById(id).orElseThrow(EventNotFoundException::new));
    }

    @Override
    public EventDetailsResponseDTO participateEvent(String id) {
        if(id == null){
            throw new IllegalArgumentException();
        }
        EventEntity event = this.repository.findById(id).orElseThrow(EventNotFoundException::new);
        List<UserEntity> list = event.getVolunteer() != null ? event.getVolunteer() : new ArrayList<>();
        UserEntity loggedUser = userService.getLoggedUser();
        if(list.stream().map(UserEntity::getId).anyMatch(userId -> loggedUser.getId().equalsIgnoreCase(userId))) {
            throw new UserAlreadyParticipate(USUARIO_JA_PARTICIPA_DO_EVENTO);
        }
        list.add(userService.getLoggedUser());
        event.setVolunteer(list);
        event = repository.save(event);
        return findEventDetails(event.getId());
    }


    @Override
    public EventDetailsResponseDTO findEventDetails(String id) {
        EventDetailsResponseDTO event = new EventDetailsResponseDTO(this.repository.findById(id).orElseThrow(EventNotFoundException::new));
        String loggedUserId = userService.getLoggedUser().getId();
        if (event.getVolunteer() != null) {
            event.setLoggedUserParticipate(event.getVolunteer()
                    .stream()
                    .anyMatch(volunteer -> loggedUserId.equalsIgnoreCase(volunteer.getId())));
        }

        if (event.getTasks() != null) {
            Long totalTasks = (long) event.getTasks().size();
            Long completedTasks = event.getTasks().stream().filter(task -> Validations.isNotNull(task.getEndDate())).count();

            double percentFinished = (completedTasks.doubleValue() / totalTasks.doubleValue()) * 100;

            double taskProgress = Double.isInfinite(percentFinished) ? 0 : BigDecimal.valueOf(percentFinished)
                    .setScale(1, RoundingMode.HALF_UP)
                    .doubleValue();

            event.setTaskProgress(taskProgress);
        }
        return event;
    }

    @Override
    public EventDetailsResponseDTO cancelParticipate(String id) {
        EventEntity event = this.repository.findById(id).orElseThrow(EventNotFoundException::new);
        String loggedUserId = userService.getLoggedUser().getId();
        event.getVolunteer().removeIf(volunteer -> loggedUserId.equalsIgnoreCase(volunteer.getId()));
        event = repository.save(event);
        return findEventDetails(event.getId());
    }

    @Override
    public EventEntity findEntityById(String id) {
        return this.repository.findById(id).orElseThrow(EventNotFoundException::new);
    }

    private void validateFields(EventRequestDTO event) {
        if (Validations.isEmptyOrisBlank(event.getTitle()))
            throw new CustomIllegalArgumentException(NOME_OBRIGATORIO);

//        if (Validations.isEmptyOrisBlank(event.getStreet()))
//            throw new CustomIllegalArgumentException(ENDERECO_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getStartDate().toString()))
            throw new CustomIllegalArgumentException(DATA_INICIAL_OBRIGATORIO);

        if (Validations.isEmptyOrisBlank(event.getEndDate().toString()))
            throw new CustomIllegalArgumentException(DATA_FINAL_OBRIGATORIO);

        if (Validations.isAfter(event.getStartDate(), event.getEndDate()))
            throw new CustomIllegalArgumentException(DATA_INICIAL_MAIOR_QUE_DATA_FINAL);

        if (Validations.isEmptyOrisBlank(event.getDescription()))
            throw new CustomIllegalArgumentException(DESCRICAO_OBRIGATORIO);

//        if (Validations.isEmptyOrisBlank(event.getLatitude()))
//            throw new CustomIllegalArgumentException(LATITUDE_OBRIGATORIO);
//
//        if (Validations.isEmptyOrisBlank(event.getLongitude()))
//            throw new CustomIllegalArgumentException(LONGITUDE_OBRIGATORIO);

        // TODO desobrir o que é EventType
//        if (event.getEventType().isBlank() || event.getEventType().isEmpty())
//            throw new CustomIllegalArgumentException(EVENTO_OBRIGATORIO);

    }

    private void uploadImages(EventRequestDTO event, EventEntity eventEntity) {
        String path = "/events/" + eventEntity.getId();
        for (int i = 0; i < event.getImages().size(); i++) {
            String key = "image_" + i;
            String uriImage = this.awsService.uploadFileS3(event.getImages().get(i), path, key);
            eventEntity.addImage(uriImage);
        }
    }
}
