package br.com.ages.mutiraodobemback.service.task;

import br.com.ages.mutiraodobemback.domain.EventEntity;
import br.com.ages.mutiraodobemback.domain.TaskEntity;
import br.com.ages.mutiraodobemback.domain.UserEntity;
import br.com.ages.mutiraodobemback.dto.TaskDTO;
import br.com.ages.mutiraodobemback.service.event.IEventService;
import br.com.ages.mutiraodobemback.service.user.IUserService;
import br.com.ages.mutiraodobemback.util.Validations;
import br.com.ages.mutiraodobemback.util.exceptions.CustomIllegalArgumentException;
import br.com.ages.mutiraodobemback.util.exceptions.UserAlreadyParticipate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static br.com.ages.mutiraodobemback.message.Message.*;
import static br.com.ages.mutiraodobemback.message.Message.NAO_FOI_POSSIVEL_REMOVER_A_TAREFA;
import static br.com.ages.mutiraodobemback.message.Message.ID_EH_OBRIGATORIO;
import static br.com.ages.mutiraodobemback.message.Message.DESCRICAO_EH_OBRIGATORIA;

@Service
public class TaskServiceImpl implements ITaskService {

    private final IUserService userService;
    private final IEventService eventService;

    @Autowired
    public TaskServiceImpl(IUserService userService, IEventService eventService) {
        this.userService = userService;
        this.eventService = eventService;
    }

    @Override
    public void createTask(String id, TaskDTO dto) {
        if(Validations.isNull(dto.getDescription()) || Validations.isEmptyOrisBlank(dto.getDescription()))
            throw new CustomIllegalArgumentException(DESCRICAO_EH_OBRIGATORIA);
        EventEntity event = this.eventService.findEntityById(id);
        List<TaskEntity> list = event.getTasks() != null ? event.getTasks() : new ArrayList<>();

        UserEntity loggedUser = userService.getLoggedUser();
        TaskEntity task = TaskEntity.builder()
                .id(String.valueOf(System.currentTimeMillis()))
                .description(dto.getDescription())
                .owner(loggedUser)
                .createDate(LocalDateTime.now())
                .build();
        list.add(task);
        event.setTasks(list);
        eventService.save(event);
    }

    @Override
    public void deleteTask(String id, TaskDTO dto) {
        if (Validations.isNull(dto.getId()))
            throw new CustomIllegalArgumentException(ID_EH_OBRIGATORIO);

        EventEntity event = this.eventService.findEntityById(id);
        String loggedUserId = userService.getLoggedUser().getId();

        boolean isEventOwner = loggedUserId.equalsIgnoreCase(event.getUser().getId());
        boolean isDeleted = event.getTasks().removeIf(task ->
                (isEventOwner || loggedUserId.equalsIgnoreCase(task.getOwner().getId()))
                && dto.getId().equalsIgnoreCase(task.getId())
        );

        if (!isDeleted)
            throw new CustomIllegalArgumentException(NAO_FOI_POSSIVEL_REMOVER_A_TAREFA);

        eventService.save(event);
    }


    @Override
    public void joinTask(String id, TaskDTO dto) {
        if (Validations.isNull(id))
            throw new CustomIllegalArgumentException(ID_EH_OBRIGATORIO);

        EventEntity event = this.eventService.findEntityById(id);
        UserEntity loggedUser = userService.getLoggedUser();

        TaskEntity taskToJoin = event.getTasks()
                .stream()
                .filter(task -> Validations.isNull(task.getVolunteer()) && dto.getId().equalsIgnoreCase(task.getId()))
                .findFirst()
                .orElseThrow(() -> new UserAlreadyParticipate(TAREFA_JA_POSSUI_VOLUNTARIO));

        taskToJoin.setVolunteer(loggedUser);
        eventService.save(event);
    }

    @Override
    public void completeTask(String id, TaskDTO dto) {
        if (Validations.isNull(id))
            throw new CustomIllegalArgumentException(ID_EH_OBRIGATORIO);

        EventEntity event = this.eventService.findEntityById(id);
        String loggedUserId = userService.getLoggedUser().getId();
        boolean isEventOwner = loggedUserId.equalsIgnoreCase(event.getUser().getId());

        TaskEntity taskToComplete = event.getTasks()
                .stream()
                .filter(task -> (isEventOwner || loggedUserId.equalsIgnoreCase(task.getOwner().getId())) && dto.getId().equalsIgnoreCase(task.getId()))
                .findFirst()
                .orElseThrow(() -> new UserAlreadyParticipate(TAREFA_NAO_PODE_SER_CONCLUIDA));

        taskToComplete.setEndDate(LocalDateTime.now());
        eventService.save(event);
    }

    @Override
    public void leaveTask(String id, TaskDTO dto) {
        EventEntity event = this.eventService.findEntityById(id);
        String loggedUserId = userService.getLoggedUser().getId();

        TaskEntity taskToLeave = event.getTasks()
                .stream()
                .filter(task -> loggedUserId.equalsIgnoreCase(task.getVolunteer().getId()) &&  dto.getId().equalsIgnoreCase(task.getId()))
                .findFirst()
                .orElseThrow(() -> new CustomIllegalArgumentException(TAREFA_NAO_PODE_SER_ABANDONADA));

        taskToLeave.setVolunteer(null);
        eventService.save(event);
    }
}
