package br.com.ages.mutiraodobemback.service.user;

import br.com.ages.mutiraodobemback.domain.UserEntity;
import br.com.ages.mutiraodobemback.dto.UserRequestDTO;
import br.com.ages.mutiraodobemback.dto.UserResponseDTO;
import br.com.ages.mutiraodobemback.repository.IUserRepository;
import br.com.ages.mutiraodobemback.service.aws.IAwsService;
import br.com.ages.mutiraodobemback.util.MD5PasswordEncoder;
import br.com.ages.mutiraodobemback.util.exceptions.CustomIllegalArgumentException;
import br.com.ages.mutiraodobemback.util.exceptions.UserAlreadyRegistered;
import br.com.ages.mutiraodobemback.util.exceptions.UserNotFoundException;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static br.com.ages.mutiraodobemback.message.Message.*;

@Service
public class UserServiceImpl implements IUserService {

    private final IUserRepository repository;
    private final IAwsService awsService;

    @Autowired
    public UserServiceImpl(IUserRepository repository, IAwsService awsService) {
        this.repository = repository;
        this.awsService = awsService;
    }

    @Override
    public UserResponseDTO save(UserRequestDTO user) {
        this.validateFields(user);

        String encryptedPassword = new MD5PasswordEncoder().encode(user.getPassword());
        UserEntity newUser = new UserEntity(user);
        newUser.setPassword(encryptedPassword);
        newUser.setRole("USER");
        newUser.setActive(true);
        newUser = this.repository.save(newUser);

        String path = "/users/" + newUser.getId();
        String uriImage = this.awsService.uploadFileS3(user.getImage(), path, "profile_image");
        newUser.setImage(uriImage);

        return new UserResponseDTO(this.repository.save(newUser));
    }

    @Override
    public List<UserResponseDTO> findAll() {
        return this.repository.findAll().stream().map(u -> UserResponseDTO.builder()
                .id(u.getId())
                .name(u.getName())
                .document(u.getDocument())
                .email(u.getEmail())
                .build()).collect(Collectors.toList());
    }

    @SneakyThrows
    @Override
    public UserEntity findByEmail(String email) {
        return this.repository.findByEmail(email).orElseThrow(UserNotFoundException::new);
    }

    @Override
    public UserEntity getLoggedUser() {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        String email = auth.getPrincipal().toString();
        return this.repository.findByEmail(email).orElse(null);
    }

    private void validateFields(UserRequestDTO user) {
        if (user.getName().isBlank() || user.getName().isEmpty())
            throw new CustomIllegalArgumentException(NOME_OBRIGATORIO);

        if (user.getDocument().isBlank() || user.getDocument().isEmpty())
            throw new CustomIllegalArgumentException(CPF_CNPJ_OBRIGATORIO);

        if (user.getPassword().isBlank() || user.getPassword().isEmpty())
            throw new CustomIllegalArgumentException(SENHA_OBRIGATORIO);

        if (user.getPhone().isBlank() || user.getPhone().isEmpty())
            throw new CustomIllegalArgumentException(TELEFONE_OBRIGATORIO);

        if (user.getEmail().isBlank() || user.getEmail().isEmpty())
            throw new CustomIllegalArgumentException(EMAIL_OBRIGATORIO);

        if (user.getImage().isBlank() || user.getImage().isEmpty())
            throw new CustomIllegalArgumentException(IMAGEM_OBRIGATORIO);

        Optional<UserEntity> userRegisteredWithEmail = this.repository.findByEmail(user.getEmail());
        if(userRegisteredWithEmail.isPresent())
            throw new UserAlreadyRegistered(EMAIL_EXISTENTE);

        Optional<UserEntity> userRegisteredWithDocument = this.repository.findByDocument(user.getDocument());
        if(userRegisteredWithDocument.isPresent())
            throw new UserAlreadyRegistered(CPF_CNPJ_EXISTENTE);
    }

}
